/*
 * $Id: DccmServer.cs 258 2007-06-02 06:54:08Z lloydd $
 */
 
using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using Mono.Unix;
using Mono.GetOptions;
using NDesk.DBus;
using org.freedesktop.DBus;

namespace Manager
{
   public class RapiForwardServer
   {
      UnixListener server;
      string path;
      string rapi_ip;
      int rapi_port;
      Thread thread;
      
      public static Dictionary<string, RapiForwardServer> List;
      
      public RapiForwardServer (string path, string rapi_ip, int rapi_port)
      {
         this.path = path;
         this.rapi_ip = rapi_ip;
         this.rapi_port = rapi_port;
         
         if (List == null)
            List = new Dictionary<string, RapiForwardServer> ();
      }
   
      public void Start ()
      {
         if (MainClass.Opt.Debug)
            Console.WriteLine ("Starting RapiForwardServer {0}", rapi_ip);

         if (List.ContainsKey (rapi_ip))
            Stop ();

         OpenServer ();
         StartThread ();
         
         List [rapi_ip] = this;

         if (MainClass.Opt.Debug)
            Console.WriteLine ("RapiForwardServer {0} saved", rapi_ip);
      }
   
      public void Stop ()
      {
         if (MainClass.Opt.Debug)
            Console.WriteLine ("Stopping RapiForwardServer {0}", rapi_ip);
         
         RapiForwardServer rfs = List [rapi_ip];
         rfs.StopThread ();
         rfs.CloseServer ();
         rfs = null;
         List.Remove (rapi_ip);
      }
      
      public void StartThread ()
      {
         thread = new Thread (new ThreadStart (Handler));
         thread.Start ();
      }
      
      public void StopThread ()
      {
         thread.Abort ();
      }
      
      public void OpenServer ()
      {
         server = new UnixListener (path);
         server.Start ();
      }
      
      public void CloseServer ()
      {
         server.Stop ();
         server = null;
      }
      
      public void Handler ()
      {
         UnixClient client = null;
         
         while (true) {

            if (MainClass.Opt.Debug)
               Console.WriteLine ("Accepting new client connection");
            
            try {
               client = server.AcceptUnixClient ();
            }
            catch (Exception e) {
               
               Console.Error.WriteLine ("RapiForwardServer closed. {0}", e.Message);
               
               break;
            }

            if (MainClass.Opt.Debug)
               Console.WriteLine ("New client connection accepted");
            
            try {
               new RapiForwarder (client, rapi_ip, rapi_port);
            }
            catch (Exception e) {
               
               Console.Error.WriteLine ("Failed to start RapiForwarder. {0}", e.Message);
            }
         }
      }
   }
}
