/*
 * $Id: DccmServer.cs 258 2007-06-02 06:54:08Z lloydd $
 */
 
using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using Mono.Unix;
using Mono.GetOptions;
using NDesk.DBus;
using org.freedesktop.DBus;

namespace Manager
{
   public class RapiForwarder
   {
      TcpClient device;
      UnixClient client;
      NetworkStream device_stream;
      NetworkStream client_stream;
      Thread ctd_thread;
      Thread dtc_thread;
      
      public RapiForwarder (UnixClient client, string rapi_ip, int rapi_port)
      {
         this.client = client;
         this.device = new TcpClient (rapi_ip, rapi_port);
         
         device_stream = this.device.GetStream ();
         client_stream = this.client.GetStream ();

         ctd_thread = new Thread (new ThreadStart (HandlerClientToDevice));
         ctd_thread.Start ();
         dtc_thread = new Thread (new ThreadStart (HandlerDeviceToClient));
         dtc_thread.Start ();
      }
      
      public void Stop ()
      {
         ctd_thread.Abort ();
         dtc_thread.Abort ();
      }
      
      public void HandlerClientToDevice ()
      {
         byte [] buf = new byte [8192];
         int readed = 0;
         
         while (true) {

            if (MainClass.Opt.Debug)
               Console.WriteLine ("Receiving data from client");
            
            try {
               // prijmy data od klienta a posli je zarizeni
               if ((readed = client_stream.Read (buf, 0, buf.Length)) > 0) {
                  
                  if (MainClass.Opt.Debug) {
                     
                     WriteBuf ("RAPI: client->device", buf, readed);
                     
                     Console.WriteLine ("Sending client data to device");
                  }
                  
                  device_stream.Write (buf, 0, readed);
               }
   
               if (readed == 0)
                  break;
            }
            catch (Exception e) {
               
               Console.Error.WriteLine (Catalog.GetString ("RAPI forwarder client->device error. {0}"), e.Message);
               
               break;
            }
         }
      }

      public void HandlerDeviceToClient ()
      {
         byte [] buf = new byte [128*1024];
         int readed = 0;

         while (true) {

            if (MainClass.Opt.Debug)
               Console.WriteLine ("Receiving data from device");
            
            try {
               // prijmy data od zarizeni a posli je klientovi
               if ((readed = device_stream.Read (buf, 0, buf.Length)) > 0) {
                  
                  if (MainClass.Opt.Debug) {
                     
                     WriteBuf ("RAPI: device->client", buf, readed);
               
                     Console.WriteLine ("Sending device data to client");
                  }
                  
                  client_stream.Write (buf, 0, readed);
               }
               
               if (readed == 0)
                  break;
            }
            catch (Exception e) {
               
               Console.Error.WriteLine (Catalog.GetString ("RAPI forwarder client->device error. {0}"), e.Message);
               
               break;
            }
         }
      }

      public void WriteBuf (string text, byte[] data, int size)
      {
         Console.WriteLine ("{0}: size: {1}", text, size);
         for (int i = 0; i < size; i++)
            Console.Write ("{0} {1}", data [i].ToString ("X2"), ((i+1) % 16 == 0 ? "\n" : ""));
         Console.WriteLine ();
         
         Console.WriteLine ("{0}: ", text);
         for (int i = 0; i < size; i++)
            Console.Write ("{0} {1}", (char) data [i], ((i+1) % 16 == 0 ? "\n" : ""));
         Console.WriteLine ();
      }
   }
}
