/*
 * $Id: TrayIconManager.cs 106 2007-04-02 09:36:46Z lloydd $
 */

using System;
using System.Runtime.InteropServices;
using Gtk;
using org.wimo.dccm;

namespace Manager
{
   public class DeviceMenuItem : ImageMenuItem
   {
      private string name;
      private IDevice device;
      
      public DeviceMenuItem (string name, IDevice device) : base (name)
      {
         this.name = name;
         this.device = device;
         
         Image = new Gtk.Image (new Gdk.Pixbuf (null, "tray-icon.png"));
      }
      
      public string DeviceName {
         get {
            return name;
         }
      }
      
      public IDevice Device {
         get {
            return device;
         }
      }
   }
   
	public class TrayIconManager
	{
      TrayIcon icon;
      
      public TrayIconManager ()
      {
         EventBox eb = new EventBox ();
         
         eb.Add (new Gtk.Image (new Gdk.Pixbuf (null, "tray-icon.png")));
         
         // hooking event
         eb.ButtonPressEvent += new ButtonPressEventHandler (this.OnImageClick);
         icon = new TrayIcon (Defines.Package);
         icon.Add (eb);
      }
      
      public void Show ()
      {
         icon.ShowAll ();
      }
      
      public void Hide ()
      {
         icon.Hide ();
      }

      private void OnImageClick (object o, ButtonPressEventArgs args) // handler for mouse click
      {
         if (MainClass.device_connection_file.Count < 1)
            return;
         
         if (args.Event.Button == 3) { //right click
            Menu popupMenu = new Menu ();
            
            foreach (string device_path in MainClass.device_connection_file.Keys) {
               IDevice dev = Device.FindInstance (device_path);
               
               DeviceMenuItem dmi = new DeviceMenuItem (
                  dev.ModelName + " '" + dev.Name + "'", dev);
               popupMenu.Add (dmi);
               dmi.Activated += new EventHandler (this.OnDeviceAtivated);
            }
            
            popupMenu.ShowAll ();
            
            popupMenu.Popup (null, null, null, args.Event.Button, args.Event.Time);
         }
      }

      private void OnDeviceAtivated(object o, EventArgs args)
      {
         DeviceMenuItem dmi = (DeviceMenuItem) o;

         if (Manager.MainClass.Opt.Debug)
            Console.WriteLine ("Show device \"{0}\" info", dmi.DeviceName);
         
         (new DeviceInfoDlg (dmi.DeviceName, dmi.Device)).Run ();
      }
	}
}
