/*
 * $Id: device-info.cs 125 2007-04-05 16:52:23Z lloydd $
 */
 
using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using Mono.Unix;
using Mono.GetOptions;
using NDesk.DBus;
using Gtk;
using Glade;
using GLib;
using org.wimo.dccm;
using WiMo.RAPI;

namespace Manager
{
	public class DeviceInfoDlg
   {
      [Widget]
      Dialog DeviceInfo;
      
      [Widget]
      Label name;

      [Widget]
      Label platform;
      
      [Widget]
      Label processor_type;

      [Widget]
      Label processor_arch;

      [Widget]
      Label model;
      
      [Widget]
      Label ip_address;
      
      [Widget]
      Label power_status;
      
      [Widget]
      ProgressBar main_battery;

      [Widget]
      ProgressBar backup_battery;

      [Widget]
      ProgressBar store_use;
      
      private IDevice device;
      private WiMo.RAPI.Connection connection;
      private WiMo.RAPI.Context ctx;
      private WiMo.RAPI.Context2 ctx2;

      public DeviceInfoDlg (string title, IDevice device)
      {
         Glade.XML gxml = new Glade.XML (null, "device-info.glade", "DeviceInfo", Defines.Package);
         gxml.Autoconnect (this);

         DeviceInfo.Title = title;
         DeviceInfo.Icon = new Gdk.Pixbuf (null, "tray-icon.png");
         
         this.device = device;
      }

      public bool Refresh ()
      {
         bool result = true;

         ISystem sys = ctx == null ? (ISystem) ctx2 : (ISystem) ctx;
         
         Dictionary<ACStatus,string> ac_status = new Dictionary<ACStatus,string> ();
         Dictionary<BatteryFlag,string> battery_flag = new Dictionary<BatteryFlag,string> ();
         
         ac_status [ACStatus.Offline] = Catalog.GetString ("Offline");
         ac_status [ACStatus.Online] = Catalog.GetString ("Online");
         ac_status [ACStatus.Backup_Power] = Catalog.GetString ("Backup Power");
         
         battery_flag [BatteryFlag.High] = Catalog.GetString ("High");
         battery_flag [BatteryFlag.Low] = Catalog.GetString ("Low");
         battery_flag [BatteryFlag.Critical] = Catalog.GetString ("Critical");
         battery_flag [BatteryFlag.Charging] = Catalog.GetString ("Charging");
         battery_flag [BatteryFlag.No_Battery] = Catalog.GetString ("No battery");
         
         try {
            SystemInfo si;
            VersionInfo ev;
            SystemPowerStatusEx p;
            StoreInformation s;
            
            name.Text = device.Name;
            model.Text = device.ModelName;
            ip_address.Text = device.DeviceIP;
            
            si = sys.CeGetSystemInfo ();
            sys.CeGetVersionEx (out ev);
            sys.CeGetSystemPowerStatusEx (out p, true);
            sys.CeGetStoreInformation (out s);
            
            processor_type.Text = si.ProcessorType.ToString ().Replace ('_', ' ');
            processor_arch.Text = si.ProcessorArch.ToString ().Replace ('_', ' ');
            platform.Text = ev.Platform.ToString ().Replace ('_', ' ');

            if (ac_status.ContainsKey (p.AcLineStatus))
               power_status.Text = ac_status [p.AcLineStatus];
            else
               power_status.Text = Catalog.GetString ("Unknown");

            if (ac_status.ContainsKey (p.AcLineStatus))
               power_status.Text = ac_status [p.AcLineStatus];
            else
               power_status.Text = Catalog.GetString ("Unknown");

            if (battery_flag.ContainsKey (p.BatteryFlag))
               main_battery.Text = battery_flag [p.BatteryFlag];
            else
               main_battery.Text = "";

            if (battery_flag.ContainsKey (p.BackupBatteryFlag))
               backup_battery.Text = battery_flag [p.BackupBatteryFlag];
            else
               backup_battery.Text = "";

            platform.Text += string.Format (" {0}.{1}.{2}", ev.MajorVersion, ev.MinorVersion, ev.BuildNumber);
            
            main_battery.Text = p.BatteryLifePercent.ToString () + " %" +
               (main_battery.Text.Length > 0 ? " - " + main_battery.Text : "");
            
            main_battery.Fraction = (double) p.BatteryLifePercent / 100;
               
            if (device.OsVersionMajor < 5) {
               
               backup_battery.Text = p.BackupBatteryLifePercent.ToString () + " %" +
                  (backup_battery.Text.Length > 0 ? " - " + backup_battery.Text : "");
               
               backup_battery.Fraction = (double) p.BackupBatteryLifePercent / 100;
            } else {
               
               double perc = p.BackupBatteryLifePercent / 255;
               int perc2 = (int) (perc * 100);
               
               backup_battery.Text = perc2.ToString () + " %" +
                  (backup_battery.Text.Length > 0 ? " - " + backup_battery.Text : "");
               
               backup_battery.Fraction = perc;
            }
            
            uint used = (s.StoreSize - s.FreeSize + 1024*1024 - 1) / (1024*1024);
            uint total = (s.StoreSize + 1024*1024 - 1) / (1024*1024);
            
            store_use.Text = string.Format ("{0}/{1} MB", used, total);
            store_use.Fraction = (double) used / (double) total;
         }
         catch (Exception e) {
            
            Console.Error.WriteLine (Catalog.GetString ("Exception: {0}"), e.Message);
            
            result = false;
         }
         
         return result;
      }
      
      public void Run ()
      {
         try {
            if (device.PasswordProvided && connection == null)
               connection = new WiMo.RAPI.Connection (device.DeviceRapiIP,
                                                 (int) device.DeviceRapiPort,
                                                 device.Password,
                                                 device.Key);
            else
               connection = new WiMo.RAPI.Connection (device.DeviceRapiIP,
                                                 (int) device.DeviceRapiPort);

            if (connection == null)
               throw new Exception ("Connection is not established");
            
            if (device.OsVersionMajor < 5 && ctx == null)
               ctx = connection.CreateContext ();
            
            if (device.OsVersionMajor >= 5 && ctx2 == null)
               ctx2 = connection.CreateContext2 ();
            
            if (Refresh ()) {
               
               DeviceInfo.Run ();
               
               DeviceInfo.Destroy ();
            }
            
            if (ctx != null)
               connection.DestroyContext (ctx);

            if (ctx2 != null)
               connection.DestroyContext2 (ctx2);
            
            connection.Dispose ();
         }
         catch (Exception e) {
            // zobrazit dialog o selhani zjisteni informaci o zarizeni
            Console.Error.WriteLine (Catalog.GetString (
               "Exception: {0}"), e.Message);
            
            MessageDialog md = new MessageDialog (null,
                                      DialogFlags.Modal,
                                      MessageType.Error,
                                      ButtonsType.Close,
                                      Catalog.GetString ("Failed to get informations about {0}"),
                                      DeviceInfo.Title);
            md.Run ();
            md.Destroy ();
         }
      }
	}
}
